#!/usr/bin/env bash

# parse log messages from qmail, disable smtp account and notfiy user
# add local (choosed country) to the ignore list of the jail
# notify admin of "banned" IP with log lines and whois output
#
# Fabian Wenk <fabian@wenks.ch>
# 12-Nov-2013	Initial publication
# 21-Nov-2013	small corrections
# 02-Dec-2013	reworked

# check if needed options are given
if [ "${3}" = "" ]; then
	echo " Error: use ${0} <jail> <ip> <logpath> <country> <url>"
	exit 1
else
	if [ ! -f ${3} ]; then
		echo " Error: file ${3} does not exist"
		exit 1
	else
		JAIL=${1}
		IP=${2}
		LOGPATH=${3}
		if [ "${4}" = "" ]; then
			echo " Error: <country> not set, use ISO country code for source IP to ignore."
			echo "        If you do not want to filter based on country use: DISABLED"
			exit 1
		else
			COUNTRY="`echo ${4} | tr '[:lower:]' '[:upper:]'`"
		fi
		if [ "${5}" = "" ]; then
			echo " Error: <url> not set."
			exit 1
		else
			URL="${5}"
		fi
	fi
fi

# Define variables
# Command to disable account (e-mail address will be appended):
COMMANDTODISABLEACCOUNT="vmoduser -d -p -w -i"
# Subject of the e-mail to the disabled account (e-mail address will be appended):
SUBJECT="SMTP access disabled for"
# Mail body text sent to the disabled account:
MAILTEXT="Dear User

We had to disable your SMTP access for your account,
for more information please visit:
${URL}

Best Regards"

# check if IP address is from a foreign country and act on it
WHOIS="`whois ${IP}`"
CURRENTCOUNTRY="`echo "${WHOIS}" | grep ^country: | awk '{ print $2 }' | tr '[:lower:]' '[:upper:]' | uniq`"
LOGLINES="`grep ${IP} ${LOGPATH}`"
ACCOUNTS="`echo "${LOGLINES}" | grep 'CHKUSER accepted sender' | awk '{ print $6 }' | sed 's/^<.*:\(.*\):>$/\1/' | sort | uniq`"
if [ ! "${CURRENTCOUNTRY}" = "${COUNTRY}" ]; then
	# disabling accounts and send info to user (and admin)
	for i in ${ACCOUNTS} ; do
		${COMMANDTODISABLEACCOUNT} ${i}
		echo "${MAILTEXT}" | mailx -s "${SUBJECT} ${i}" ${i}
	done
	echo "E-mail accounts disabled:"
	echo "========================="
	echo
	echo "Because of abuse from ${IP} the following accounts had been disabled:"
	echo
	echo "${ACCOUNTS}"
else
	echo "No e-mail account disabled, IP address added to ignorelist:"
	echo "==========================================================="
fi
echo
echo "Affected IP address: ${IP}"
echo "Affected Country:    ${CURRENTCOUNTRY}"
if [ "${CURRENTCOUNTRY}" = "${COUNTRY}" ]; then
	echo
	fail2ban-client set ${JAIL} addignoreip ${IP}
	echo
	echo "To manually disable this e-mail accounts, use:"
	for i in ${ACCOUNTS} ; do
		echo "  ${COMMANDTODISABLEACCOUNT} ${i}"
	done
	echo
	echo "Do not forget to inform the user!"
fi
echo
echo
echo "Entries from log file:"
echo "----------------------"
echo "${LOGLINES}"
echo
echo
echo "Whois output:"
echo "-------------"
echo "${WHOIS}"
exit 0
